use std::fmt::Display;

use viperid::VResult;
use wasmi::{core::{HostError, Trap}, Caller, Extern, Linker};

use crate::executor::ExecutorState;

pub(crate) fn integrate(linker: &mut Linker<ExecutorState>) -> VResult<()> {
    linker.func_wrap("viperid", "Pset", pset)?;

    linker.func_wrap("viperid", "YieldFrame", yield_frame)?;

    Ok(())
}

fn pset(mut caller: Caller<ExecutorState>, x: i32, y: i32, color: i32) -> Result<(), Trap> {
    let memory = match caller.get_export("memory") {
        Some(Extern::Memory(m)) => m,
        _ => return Err(wasmi::core::Trap::new(String::from("missing required memory export"))),
    };
    let(_, ctx) = memory.data_and_store_mut(&mut caller);
    ctx.device.shared.screen.pset(x, y, (color & 0xff) as u8);
    Ok(())
}

fn yield_frame(_caller: Caller<ExecutorState>) -> Result<(), Trap> {
    Err(Trap::from(YieldFrame {}))
}

#[derive(Debug)]
pub(crate) struct YieldFrame {

}

impl Display for YieldFrame {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.write_str("<device wants to display a frame>")
    }
}

impl HostError for YieldFrame {

}
