use std::cell::RefCell;

use crate::PALETTE;

pub const SCREEN_W: usize = 640;
pub const SCREEN_H: usize = 480;

const PIXEL_SCALAR: usize = 4;
// const CURSES_SNAP: usize = 8;

pub struct Screen {
    // NOTE: There are better data structures we could use than this
    pixels: RefCell<Vec<u8>>
}

impl Screen {
    pub fn new() -> Screen {
        Screen {
            pixels: RefCell::new(vec![0; SCREEN_W * SCREEN_H])
        }
    }

    pub fn to_bitmap(&self, bitmap: &mut Vec<u32>) {
        bitmap.resize(SCREEN_W * SCREEN_H, 0);

        let pxl = self.pixels.borrow();
        for i in 0..pxl.len() {
            bitmap[i] = PALETTE[pxl[i] as usize];
        }
    }

    pub fn pset(&self, x: i32, y: i32, color: u8) {
        let px0 = x * PIXEL_SCALAR as i32;
        let py0 = y * PIXEL_SCALAR as i32;

        if px0 < 0 || py0 < 0 || px0 >= SCREEN_W as i32 || py0 >= SCREEN_H as i32 {
            return;
        }

        let px0 = px0 as usize;
        let py0 = py0 as usize;

        let mut pxl = self.pixels.borrow_mut();
        for py in py0..py0 + PIXEL_SCALAR {
            for px in px0..px0 + PIXEL_SCALAR {
                pxl[py * SCREEN_W + px] = color;
            } 
        }
    }
}